<?php

namespace App\Http\Controllers;

use Carbon\Carbon;
use Carbon\CarbonPeriod;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Http;
use Illuminate\Support\Facades\Cache;

class StatsController extends Controller
{

    public function index(Request $request)
    {
        try {
            $from = $request->has('from')
                ? Carbon::parse($request->from)->startOfDay()->setTimezone('UTC')
                : Carbon::now()->startOfDay()->setTimezone('UTC');

            $to = $request->has('to')
                ? Carbon::parse($request->to)->endOfDay()->setTimezone('UTC')
                : Carbon::now()->endOfDay()->setTimezone('UTC');

            $countries = config('countries');
            $stats = [
                "global" => [
                    'paid' => 0,
                    'refunded' => 0,
                    'draft' => 0,
                    'draftWithData' => 0,
                    'pendingPayment' => 0,
                    'moneyInvoiced' => 0,
                    'moneyRefunded' => 0,
                    'ultraFast' => 0,
                    'normal' => 0,
                    'payment_gateway' => []
                ],
                "last7Days" => [],
                "yesterdayByHour" => [],
                "todayByHour" => [],
                "yesterdayPaymentsByCountry" => [],
                "todayPaymentsByCountry" => [],
                "yesterdayEarningsByCountry" => [],
                "todayEarningsByCountry" => [],
            ];

            $globalResponses = Cache::remember('stats-global-' . $from->copy()->format('Y-m-d') . '-' . $to->copy()->format('Y-m-d'), 60 * 5, function () use ($countries, $from, $to) {
                $responses = Http::pool(function ($pool) use ($countries, $from, $to) {
                    foreach ($countries as $iso => $country) {
                        $pool->as($iso)->get($country['url'] . '/api/stats', [
                            'from' => $from->toISOString(),
                            'to' => $to->toISOString(),
                        ]);
                    }
                });

                $results = [];

                foreach ($responses as $iso => $response) {
                    try {
                        $results[$iso] = $response instanceof \Illuminate\Http\Client\Response && $response->successful() ? $response->json() : null;
                    } catch (\Exception $e) {
                    }
                }

                return $results;
            });

            foreach ($globalResponses as $iso => $data) {
                if ($data == null) {
                    continue;
                }

                $stats[$iso] = $data;

                foreach (array_keys($stats["global"]) as $key) {
                    if ($key === 'payment_gateway') {
                        foreach ($data[$key] as $value) {
                            if (!isset($stats["global"][$key][$value['payment_gateway']])) {
                                $stats["global"][$key][$value['payment_gateway']] = $value["total"];

                                continue;
                            }

                            $stats["global"][$key][$value['payment_gateway']] += $value["total"];
                        }
                    } else {
                        $stats["global"][$key] += $data[$key] ?? 0;
                    }
                }
            }

            $period = CarbonPeriod::create(Carbon::now('UTC')->subDays(7), Carbon::now('UTC'));

            foreach ($period as $date) {
                $stats["last7Days"][$date->format('Y-m-d')] = [];
            }

            $last7DaysResponses = Cache::remember('stats-last7days-' . Carbon::now('UTC')->format('Y-m-d'), 60 * 1440, function () use ($countries, $stats) {
                $responses = Http::pool(function ($pool) use ($countries, $stats) {
                    $formattedToday = Carbon::now('UTC')->format('Y-m-d');

                    foreach ($countries as $iso => $country) {
                        foreach ($stats["last7Days"] as $date => $value) {
                            if ($date === $formattedToday) {
                                continue;
                            }

                            $pool->as("$iso*$date")->get($country['url'] . '/api/stats', [
                                'from' => Carbon::parse($date)->startOfDay()->setTimezone('UTC')->toISOString(),
                                'to' => Carbon::parse($date)->endOfDay()->setTimezone('UTC')->toISOString()
                            ]);
                        }
                    }
                });

                $results = [];

                foreach ($responses as $key => $response) {
                    [$iso, $date] = explode('*', $key);

                    try {
                        $results[$iso][$date] = $response instanceof \Illuminate\Http\Client\Response && $response->successful() ? $response->json() : null;
                    } catch (\Exception $e) {
                    }
                }

                return $results;
            });

            $todayResponses = Cache::remember('stats-today' . Carbon::now('UTC')->format('Y-m-d'), 60 * 5, function () use ($countries) {
                $responses = Http::pool(function ($pool) use ($countries) {
                    foreach ($countries as $iso => $country) {
                        $pool->as($iso)->get($country['url'] . '/api/stats', [
                            'from' => Carbon::now('UTC')->startOfDay()->setTimezone('UTC')->toISOString(),
                            'to' => Carbon::now('UTC')->endOfDay()->setTimezone('UTC')->toISOString()
                        ]);
                    }
                });

                $results = [];

                foreach ($responses as $iso => $response) {
                    try {
                        $results[$iso] = $response instanceof \Illuminate\Http\Client\Response && $response->successful() ? $response->json() : null;
                    } catch (\Exception $e) {
                    }
                }

                return $results;
            });

            $formattedToday = Carbon::now('UTC')->format('Y-m-d');

            foreach ($todayResponses as $iso => $data) {
                if ($data == null) {
                    continue;
                }

                $last7DaysResponses[$iso][$formattedToday] = $data;
            }

            foreach ($last7DaysResponses as $iso => $data) {
                foreach ($data as $date => $value) {
                    $stats["last7Days"][$date][$iso] = $value;
                }
            }

            for ($i = 0; $i < 24; $i++) {
                $hour = $i < 10 ? '0' . $i : $i;

                $stats["yesterdayByHour"][$hour] = [];
                $stats["todayByHour"][$hour] = [];
            }

            $yesterdayByHourResponses = Cache::remember('stats-yesterday-by-hour-' . Carbon::now('UTC')->subDay()->format('Y-m-d'), 60 * 1440, function () use ($countries, $stats) {
                $responses = Http::pool(function ($pool) use ($countries, $stats) {
                    foreach ($countries as $iso => $country) {
                        foreach (array_keys($stats["yesterdayByHour"]) as $hour) {
                            $pool->as("$iso*$hour")->get($country['url'] . '/api/stats', [
                                'from' => Carbon::yesterday('UTC')->setTime($hour, 0, 0)->toISOString(),
                                'to' => Carbon::yesterday('UTC')->setTime($hour, 59, 59)->toISOString()
                            ]);
                        }
                    }
                });

                $results = [];

                foreach ($responses as $key => $response) {
                    [$iso, $hour] = explode('*', $key);

                    try {
                        $results[$iso][$hour] = $response instanceof \Illuminate\Http\Client\Response && $response->successful() ? $response->json() : null;
                    } catch (\Exception $e) {
                    }
                }

                return $results;
            });

            foreach ($yesterdayByHourResponses as $iso => $data) {
                foreach ($data as $hour => $value) {
                    $stats["yesterdayByHour"][$hour][$iso] = $value;
                }
            }

            $todayByHourResponses = Cache::remember('stats-today-by-hour-' . Carbon::now('UTC')->format('Y-m-d:H'), 60 * 1440, function () use ($countries, $stats) {
                $responses = Http::pool(function ($pool) use ($countries, $stats) {
                    $currentHour = str_pad(Carbon::now('UTC')->hour, 2, "0", STR_PAD_LEFT);

                    foreach ($countries as $iso => $country) {
                        foreach (array_keys($stats["todayByHour"]) as $hour) {
                            if (intval($hour) >= intval($currentHour)) {
                                continue;
                            }

                            $pool->as("$iso*$hour")->get($country['url'] . '/api/stats', [
                                'from' => Carbon::now('UTC')->setTime($hour, 0, 0)->toISOString(),
                                'to' => Carbon::now('UTC')->setTime($hour, 59, 59)->toISOString()
                            ]);
                        }
                    }
                });

                $results = [];

                foreach ($responses as $key => $response) {
                    [$iso, $hour] = explode('*', $key);

                    try {
                        $results[$iso][$hour] = $response instanceof \Illuminate\Http\Client\Response && $response->successful() ? $response->json() : null;
                    } catch (\Exception $e) {
                    }
                }

                return $results;
            });

            $currentHourResponses = Cache::remember('stats-current-hour-' . Carbon::now('UTC')->format('Y-m-d:H'), 60 * 5, function () use ($countries) {
                $responses = Http::pool(function ($pool) use ($countries) {
                    $currentHour = str_pad(Carbon::now('UTC')->hour, 2, "0", STR_PAD_LEFT);

                    foreach ($countries as $iso => $country) {
                        $pool->as($iso)->get($country['url'] . '/api/stats', [
                            'from' => Carbon::now('UTC')->setTime($currentHour, 0, 0)->toISOString(),
                            'to' => Carbon::now('UTC')->setTime($currentHour, 59, 59)->toISOString()
                        ]);
                    }
                });

                $results = [];

                foreach ($responses as $iso => $response) {
                    try {
                        $results[$iso] = $response instanceof \Illuminate\Http\Client\Response && $response->successful() ? $response->json() : null;
                    } catch (\Exception $e) {
                    }
                }

                return $results;
            });

            $currentHour = str_pad(Carbon::now('UTC')->hour, 2, "0", STR_PAD_LEFT);

            foreach ($currentHourResponses as $iso => $data) {
                if ($data == null) {
                    continue;
                }

                $todayByHourResponses[$iso][$currentHour] = $data;
            }

            foreach ($todayByHourResponses as $iso => $data) {
                foreach ($data as $hour => $value) {
                    $stats["todayByHour"][$hour][$iso] = $value;
                }
            }

            $yesterdayPayments = 0;
            $yesterdayEarnings = 0;
            $todayPayments = 0;
            $todayEarnings = 0;

            for ($i = 0; $i <= intval($currentHour); $i++) {
                $hour = str_pad($i, 2, "0", STR_PAD_LEFT);

                if (isset($stats["yesterdayByHour"][$hour])) {
                    foreach ($stats["yesterdayByHour"][$hour] as $key => $data) {
                        $yesterdayEarnings += $data['moneyInvoiced'] ?? 0;

                        $yesterdayPayments += $data['paid'] ?? 0;

                        if (!isset($stats["yesterdayPaymentsByCountry"][$key])) {
                            $stats["yesterdayPaymentsByCountry"][$key] = 0;
                        }

                        if (!isset($stats["yesterdayEarningsByCountry"][$key])) {
                            $stats["yesterdayEarningsByCountry"][$key] = 0;
                        }

                        $stats["yesterdayEarningsByCountry"][$key] += $data['moneyInvoiced'] ?? 0;

                        $stats["yesterdayPaymentsByCountry"][$key] += $data['paid'] ?? 0;
                    }
                }

                if (isset($stats["todayByHour"][$hour])) {
                    foreach ($stats["todayByHour"][$hour] as $key => $data) {
                        $todayEarnings += $data['moneyInvoiced'] ?? 0;

                        $todayPayments += $data['paid'] ?? 0;

                        if (!isset($stats["todayPaymentsByCountry"][$key])) {
                            $stats["todayPaymentsByCountry"][$key] = 0;
                        }

                        if (!isset($stats["todayEarningsByCountry"][$key])) {
                            $stats["todayEarningsByCountry"][$key] = 0;
                        }

                        $stats["todayEarningsByCountry"][$key] += $data['moneyInvoiced'] ?? 0;

                        $stats["todayPaymentsByCountry"][$key] += $data['paid'] ?? 0;
                    }
                }
            }

            if ($yesterdayEarnings > 0) {
                $percentageDifference = 100 - (($todayEarnings * 100) / $yesterdayEarnings);
                $percentageDifference = $yesterdayEarnings > $todayEarnings ? abs($percentageDifference) * -1 : abs($percentageDifference);
            } else {
                $percentageDifference = $todayEarnings > 0 ? 100 : 0;
            }

            $stats["percentageDifference"] = round($percentageDifference, 2);
            $stats["yesterdayPayments"] = $yesterdayPayments;
            $stats["yesterdayEarnings"] = $yesterdayEarnings;

            return view('auth.stats', [
                'from' => $from,
                'to' => $to,
                'stats' => $stats,
            ]);
        } catch (\Exception $e) {
            return dd($e);
            abort(404);
        }
    }
}
